from pathlib import Path
from os import environ

NAMESPACE_STATUS = {
    'now': 'argocd',
    'previous': 'default'
}

homedir = str(Path.home())
env_kubeconfig = environ.get('KUBECONFIG')
path_kubeconfig = '/.kube/config'



# Instantiate your kubernetes class and pass in config
from ksns.client import K8s
from os import environ

def list_pods():
    kubeconfig = environ.get('KUBECONFIG')
    if kubeconfig is None:
        kubeconfig = '~/.kube/config'

    kube = K8s(configuration_yaml=kubeconfig)
    pods = kube.client.list_pod_for_all_namespaces(watch=False)
    print(pods)
    return "pods"

if __name__ == "__main__":
    list_pods()


import click, yaml, errno, os

from ksns.client import K8s
from ksns.const import homedir, env_kubeconfig, path_kubeconfig, NAMESPACE_STATUS

@click.group()
def main():
    """ 
NOTE: Set the KUBECONFIG environment variable or ~/.kube/config will be considered

USAGE: 

        1. list namespaces : list

        2. switch namespaces: ns <namespace_name>
    
    """
    pass


@main.command()
def list():
    """ List of namespaces in context """
    if not env_kubeconfig:
        kubeconfig = homedir + path_kubeconfig
    else:
        kubeconfig = env_kubeconfig
    if not os.path.exists(kubeconfig):
        click.secho(f"kubeconfig not found in path '{kubeconfig}'", fg='red', nl=True)
        raise click.Abort()

    try:
        kctx = K8s(configuration_yaml=kubeconfig)
        list_namespaces = kctx.client.list_namespace(watch=False)
        current_namespace = kctx.config['contexts'][0]['context']['namespace']
        for i in list_namespaces.items:
            namespace = i.metadata.name
            if namespace == current_namespace:
                fg = 'yellow'
            else: 
                fg = 'green'
            click.secho(namespace, fg=fg, nl=True)

        click.secho(f"using config: {kubeconfig}", fg='blue', nl=True)

    except Exception as error:
        click.secho(f"Caught this error: '{repr(error)}'", fg='red', nl=True)

@main.command()
@click.argument('namespace', type=str, default='default')
def ns(namespace):
    """ Switch to another namespace: <namespace_name> """
    namespace_arr = []
    if not env_kubeconfig:
        kubeconfig = homedir + path_kubeconfig
    else:
        kubeconfig = env_kubeconfig
    if not os.path.exists(kubeconfig):
        click.secho(f"kubeconfig not found in path '{kubeconfig}'", fg='red', nl=True)
        raise click.Abort()

    try:
        kctx = K8s(configuration_yaml=kubeconfig)
        list_namespaces = kctx.client.list_namespace(watch=False)
        for i in list_namespaces.items:
            namespace_arr.append(i.metadata.name)
        if namespace not in namespace_arr:
            click.secho(f"Specified namespace '{namespace}' is not valid.", fg='red', nl=True)
            raise click.Abort()
        current_namespace = kctx.config['contexts'][0]['context']['namespace']
        with open(kubeconfig) as file:
            getconfig = yaml.load(file, Loader=yaml.FullLoader)

        with open(kubeconfig, 'w') as config_file:
            getconfig['contexts'][0]['context']['namespace'] = namespace
            sort_file = yaml.dump(getconfig, config_file, sort_keys=True)
        
        click.secho(f"switched to {namespace} namespace", fg='green', nl=True)
        click.secho(f"using config: {kubeconfig}", fg='blue', nl=True)

    except Exception as error:
        click.secho(f"Caught this error: '{repr(error)}'", fg='red', nl=True)
        
    

if __name__ == "__main__":
    main()


from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.2'
DESCRIPTION = 'Switch kubernetes namespace'
LONG_DESCRIPTION = 'A package that allows to switch kubernetes namespaces easily.'

# Setting up
setup(
    name="ksns",
    version=VERSION,
    py_modules=['ksns'],
    include_package_data=True,
    author="Ratul Basak",
    author_email="ratulbasak93@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/ratulbasak/ksns",
    packages=find_packages(),
    install_requires=['click', 'kubernetes', 'pyyaml'],
    entry_points='''
        [console_scripts]
        ksns=ksns.cli:main
    ''',
    keywords=['python', 'kubernetes', 'namespace', 'switch namespace', 'k8s', 'package'],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.9",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


import yaml

from kubernetes import client
from kubernetes.client import Configuration
from kubernetes.config import kube_config


class K8s(object):
    def __init__(self, configuration_yaml):
        self.configuration_yaml = configuration_yaml
        self._configuration_yaml = None

    @property
    def config(self):
        with open(self.configuration_yaml, 'r') as f:
            if self._configuration_yaml is None:
                self._configuration_yaml = yaml.load(f, Loader=yaml.CLoader)
        return self._configuration_yaml
            

    @property
    def client(self):
        k8_loader = kube_config.KubeConfigLoader(self.config)
        call_config = type.__call__(Configuration)
        k8_loader.load_and_set(call_config)
        Configuration.set_default(call_config)
        return client.CoreV1Api()
